@@ -1,5 +1,5 @@
 suppressPackageStartupMessages(library(stars))
-demo(nc, echo = FALSE, ask = FALSE)
+nc = read_sf(system.file("gpkg/nc.gpkg", package="sf"))
 (x = st_rasterize(nc)) # default grid:
 plot(x, axes = TRUE)
 # a bit more customized grid:
@@ -7,7 +7,9 @@ plot(x, axes = TRUE)
 plot(x, axes = TRUE)
 (ls = st_sf(a = 1:2, st_sfc(st_linestring(rbind(c(0.1, 0), c(1.1, 1))), st_linestring(rbind(c(0, 0.05), c(1, 0.05))))))
 (grd = st_as_stars(st_bbox(ls), nx = 10, ny = 10, xlim = c(0, 1.0), ylim = c(0, 1), values = NA_real_))
-# the following two plots suggests a half-gridcell-shift problem:
+# the following two plots confirm that (only) the
+# upper-left corner is part of the grid cell (when dy is negative), leading 
+# to a seemingly half-gridcell-shift problem:
 sf_extSoftVersion()["GDAL"]
 plot(st_rasterize(ls, grd), axes = TRUE, reset = FALSE) # ALL_TOUCHED=FALSE;
 plot(ls, add = TRUE, col = "red")
